@#define IRF_periods=80
@#define CMAES=0

var 
Y           % final output
Ym          % intermediary output
L           % labor hours
w           % real wage
C           % consumption
U_c         % marginal utility of consumption
Lambda      % stochastic discount factor of households 
I           % investment
K           % capital
Q           % price of capital
CRR
CR
CN
Pm          % price of intermediary goods (real marginal cost for final goood producer)
infl        % inflation
inflstar    % optimal price divided by past price level
F           % auxiliary variable for price setting
Z           % auxiliary variable for price setting
Dis         % price dispersion
Rd          % real deposit rate 
i           % nominal interest rate
Rk          % real rate of capital
Rb          % real rate of bonds held by banks
Rp          % real rate of portfolio by banks
ERk         % expected real rate of capital
ERb         % expected real rate of bonds held by banks
prem        % premium of return on capital - return on deposits
prem2       % premium of return on bonds - return on deposits
Phi         % Leverage ratio of banks
portf_B     % portfolio of banks
N           % net worth of banks
Om          % portfolio share of capital assets
D           % economy-wide deposits
nu_k        % value of having another unit of capital assets
nu_b        % value of having another unit of bonds (for banks)
nu_n        % value of having another unit of net worth
G           % government spending
ghat           % government spending (shock process)
Gy          % government spending share of output
T           % taxes
B          % government debt
log_y
log_C
log_I
log_B;
%capital quality, government spending, interest rate, technology, and net worth shock
varexo  eps_g  ;
parameters 
    beta hh delta varphi eta_i alpha gam epsilon kappa_pi kappa_y G_over_Y B_over_Y theta lambda chi
    rho_i rho_ksi rho_a rho_g sigma_i sigma_ksi sigma_a sigma_g sigma_n
    Y_ss L_ss w_ss U_c_ss Y_over_K L_over_K K_ss Y_ss Ym_ss G_ss I_ss C_ss Z_ss F_ss B_ss N_ss D_ss T_ss
    a_ss ksi_ss g_ss Lambda_ss Pm_ss Phi_ss Q_ss Dis_ss infl_ss inflstar_ss Rd_ss Rp_ss Rb_ss Rk_ss i_ss
    prem_ss nu_b_ss nu_n_ss nu_k_ss portf_B_ss Om_ss Gy_ss root_g_1    
    root_g_2   CRR_ss CR_ss CN_ss kappac v mu;
    
% standard Parameters
v=0.45;
mu=0.3;
beta=0.99;          % discount factor 
hh=0.815;           % habit formation
delta = 0.025;      % depreciation rate
varphi = 0.276;     % inverse of Frisch elasticity
eta_i = 1.728;      % inv. adjustment cost parameter
alpha = 0.33;       % output elasticity of capital
gam=0.779;          % Calvo parameter
epsilon=4.167;      % elasiticity of substitution between varieties of final goods
G_over_Y=0.2;       % stst. government spending share of output
B_over_Y=2.4;       % stst. debt/GDP
Phi_ss=4;           % stst. Leverage ratio of banks
theta=1-1/16;       % survival rate of banks
kappa_pi=1.50000000;% Taylor rule coefficient on inflation
kappa_y =0.12500000;% Taylor rule coefficient on output growth
%shock parameters
rho_i=0.8;        % interest rate smoothing
rho_ksi=0.66;     % persistence of capital quality shock
rho_a=0.95;       % persistence of technology shock
rho_g=0.80;       % persistence of government spending shock
sigma_ksi=0.05;   % std. dev. of capital quality shock
sigma_a=0.01;     % std. dev. of technology shock
sigma_g=0.05;     % std. dev. of government spending shock
sigma_i=0.0025;   % std. dev. of interest rate shock
sigma_n=0.01;     % std. dev. of net worth shock
%%% steady state values of variables labelled above
Lambda_ss = 1;                     %
Q_ss=1;                            % 
Dis_ss=1;
infl_ss=1;
inflstar_ss=1;
a_ss=1;
g_ss=1;
ksi_ss=1;
Pm_ss = (epsilon-1)/epsilon;       %
prem_ss = 0.01/4;
prem2_ss = prem_ss;
Phi_ss=4;
Rd_ss=1/beta;
i_ss=Rd_ss;
Rk_ss=Rd_ss+prem_ss;
Rb_ss=Rk_ss;
Rp_ss=Rk_ss;
ERk_ss=Rk_ss;
ERb_ss=Rk_ss;
nu_n_ss = (1-theta)*beta*(Rd_ss)/(1-theta*beta);
nu_k_ss = (1-theta)*beta*(Rk_ss-Rd_ss)/(1-theta*beta);
nu_b_ss = nu_k_ss; 
lambda = nu_k_ss + nu_n_ss/Phi_ss;          % tightness parameter on incentive constraint
chi = 1-theta*((Rk_ss-Rd_ss)*Phi_ss+Rd_ss); % share of old net worth that is given to new bankers
w_ss = (alpha^(alpha)*(1-alpha)^(1-alpha)*Pm_ss*(Rk_ss-1+delta)^(-alpha))^(1/(1-alpha));
Y_over_K = (Rk_ss-1+delta)/(alpha*Pm_ss);
L_over_K = (Y_over_K)^(1/(1-alpha));
I_over_Y = delta*(Y_over_K)^(-1);
C_over_Y = 1-G_over_Y-I_over_Y;
habit = (1-beta*hh)/(1-hh);
Y_ss = (L_over_K)^(-alpha*varphi/(1+varphi))*(habit*w_ss*C_over_Y^(-1))^(1/(1+varphi));
Ym_ss= Y_ss;
C_ss = Y_ss*C_over_Y;
G_ss = Y_ss*G_over_Y;
Gy_ss = G_over_Y;
I_ss = Y_ss*I_over_Y;
K_ss = I_ss/delta;
L_ss = L_over_K*K_ss;

CN_ss=w_ss*L_ss;
CR_ss=(C_ss-mu*CN_ss)/(1-mu);
kappac  = G_ss^(-1/v) / (CR_ss^(-1/v) + G_ss^(-1/v));
CRR_ss  = kappac*(CR_ss-hh*CR_ss)^((v-1)/v)+(1-kappac)*G_ss^((v-1)/v);
U_c_ss     = (((1/CRR_ss)*kappac*(CR_ss*(1-hh))^(-1/v))*(1-beta*hh));


%U_c_ss = habit/C_ss;
Z_ss = 1/(1-beta*gam)*Y_ss;
F_ss = Z_ss*Pm_ss;
B_ss = Y_ss*B_over_Y;
N_ss = (K_ss+B_ss)/Phi_ss;
Om_ss = K_ss/(K_ss+B_ss);
Rp_ss = Om_ss*(Rk_ss)+(1-Om_ss)*Rb_ss;
T_ss = (Rb_ss-1)*B_ss+G_ss;
portf_B_ss = Phi_ss*N_ss;
D_ss = K_ss + B_ss - N_ss;
root_g_1=0.9602;
root_g_2=0;
model;
% Households
# rho_g_1= (root_g_1+root_g_2);
# rho_g_2= - root_g_1*root_g_2;


CRR= kappac*(CR-hh*CR(-1))^((v-1)/v)+(1-kappac)*G^((v-1)/v);   
U_c =(1/CRR)*kappac*(CR-hh*CR(-1))^(-1/v)-beta*hh*(1/CRR(+1))*kappac*(CR(+1)-hh*CR)^(-1/v);
CN=w*L;
C=(1-mu)*CR+mu*CN;

Lambda  =   U_c/U_c(-1);                                        % Stochastic discount rate
beta*Rd(+1)*Lambda(+1)  =   1;                                       % Euler equation
L^varphi    = U_c*w;                                            % Labor supply
% Intermediate goods producer
Rk     =   (Pm*alpha*Ym/K(-1)+Q*(1-delta))/Q(-1); % Return on capital (Assumption: this = E26)
Ym     =   (K(-1))^alpha*L^(1-alpha); % Production function
w     =   Pm*(1-alpha)*Ym/L; % Real Wages (Assumption: This replaces E28 (Eq. for mc))
% Capital Goods Producer

1/Q =   1 - eta_i/2*(I/I(-1)-1)^2 
               - eta_i*(I/I(-1)-1)*I/I(-1) 
               + beta*Lambda(+1)*eta_i*(I(+1)/I-1)*(I(+1)/I)^2*Q/Q(+1); //E29
K  =   (1-delta)*K(-1) + (1 - eta_i/2*(I/I(-1)-1)^2)*I;          % Capital accumulation equation
% Retailer
Ym    =   Y*Dis;                                                                             % Retail output
Dis    =   gam*Dis(-1)*infl^epsilon + 
                (1-gam)*((1-gam*infl^(epsilon-1))/(1-gam))^(-epsilon/(1-epsilon));                     % Price dispersion
F       =   Y*Pm + beta*gam*Lambda(+1)*infl(+1)^epsilon    *F(+1);            % Optimal price choice
Z       =   Y         + beta*gam*Lambda(+1)*infl(+1)^(epsilon-1)*Z(+1);            % Optimal price choice
inflstar   =  epsilon/(epsilon-1)*F/Z*infl;                                             % Optimal price choice
infl^(1-epsilon)     =   gam + (1-gam)*(inflstar)^(1-epsilon);                                    % Price index

% Financial Intermediaries
nu_k=beta*Lambda(+1)*((Rk(+1)-Rd(+1))*(1-theta)+ 
          theta*Q(+1)*K(+1)/(Q*K)*nu_k(+1)); % shadow excess value of loans
nu_b=beta*Lambda(+1)*((Rb(+1)-Rd(+1))*(1-theta)+
          theta*B(+1)/(B)*nu_b(+1));     % shadow value of bonds
nu_n=beta*Lambda(+1)*(Rd(+1)*(1-theta) +
          theta*(N(+1)/N)*nu_n(+1));     % shadow value of equity
nu_k=nu_b;                                    % relation between shadow values of loans and bonds
Phi=nu_n/(lambda-nu_k);                  % Leverage of Banks
N= theta*((Rp-Rd)*Phi(-1) + Rd)*N(-1) + chi* N(-1);    % law of motion for net worth
N+D= Q*K+B;                    % balance sheet identity of banks
portf_B = Q*K+B;                    % Banks' portfolio
Q*K = Om * Phi * N;            % claims on capital assets

B        = (1 - Om) * Phi * N;      % claims on bonds 
ERk= Rk(+1);                                  % Expected Return on Capital
ERb = Rb(+1);                                 % Expected Return on Bonds
prem =   Rk(+1)-Rd(+1);                  % Loan Premium
prem2 =   Rb(+1)-Rd(+1);                 % Bond Premium
Rp = Rk*Om(-1)+Rb*(1-Om(-1));  % Return on portfolio

% Fiscal policy
%G = (1-rho_g)*G_ss + rho_g*G(-1) + eg; %
B = Rb*B(-1)+G-T; % Government Budget Constraint
G = G_ss*exp(ghat); % Government consumption
T = T_ss; % Taxes
Gy = G/Y_ss; % Share of gov. consumption in GDP
 
% Additional Equations
Y =   C +G + I; % Aggregate resource constraint
i(-1) =   Rd*infl; % Fisher equation
i =   i(-1)^rho_i* (i_ss*infl^kappa_pi*((Y)/(Y(-1)))^(kappa_y))^(1-rho_i); % Taylor rule

% Shocks

ghat  =   rho_g_1*ghat(-1)+eps_g; %44. Government consumption shock
%ghat=rho_g_1*ghat(-1)+rho_g_2*ghat(-2)+eps_g;

log_y=log(Y);
log_C=log(C);
log_I=log(I);
log_B=log(B);
end;


initval;
CRR=CRR_ss;
CR=CR_ss;
CN=CN_ss;
Y=Y_ss;
Ym=Ym_ss;
K=K_ss;
L=L_ss;
I=I_ss;
C=C_ss;
G=G_ss;
Q=Q_ss;
U_c=U_c_ss;
Lambda=1;
Rk=Rk_ss;
Rd=Rd_ss;
N=N_ss;
Pm=Pm_ss;
w=w_ss;
Dis=Dis_ss;
F=F_ss;
Z=Z_ss;
i=i_ss;
prem=prem_ss;


ghat=0;
infl=1;
inflstar=1;
B=B_ss;
D=D_ss;
T=T_ss;
Phi=Phi_ss;
Rb=Rb_ss;
nu_k=nu_k_ss;
nu_b=nu_b_ss;
nu_n=nu_n_ss;
prem2=prem2_ss;
ERk=Rk_ss;
ERb=Rb_ss;
Rp=Rp_ss;
portf_B=portf_B_ss;
Om=Om_ss;
Gy=Gy_ss;
log_y=log(Y_ss);
log_C=log(C_ss);
log_I=log(I_ss);
log_B=log(B_ss);
end;

resid(1);
steady;
check;

shocks;
%var e_a=sigma_a^2;
%var e_ksi=sigma_ksi^2;
%var e_g=sigma_g^2;
var eps_g=1;
%var e_i=sigma_i^2;
end;

stoch_simul(order = 1,irf=@{IRF_periods}) ghat log_y   log_I log_C;

%% get empirical IRFs and weighting matrix
[IRF_empirical,IRF_weighting,IRF_quantiles]=get_empirical_IRFs(@{IRF_periods})
         

x_start=[root_g_1 root_g_2]; %use calibration as starting point


%make sure Dynare does not print out stuff during runs
options_.nomoments=1;
options_.nofunctions=1;
options_.nograph=1;
options_.verbosity=0;

%set noprint option to suppress error messages within optimizer
options_.noprint=1;

@#if CMAES==0
    % set csminwel options
    H0 = 1e-2*eye(length(x_start)); %Initial Hessian 
    crit = 1e-8; %Tolerance
    nit = 1000;  %Number of iterations

    [fhat,x_opt_hat] = csminwel(@IRF_matching_objective,x_start,H0,[],crit,nit,IRF_empirical,IRF_weighting);
@#else
    %set CMAES options
    H0=0.2*ones(size(x_start,1),1)
    cmaesOptions = options_.cmaes;
    cmaesOptions.LBounds = [-1;-1];
    cmaesOptions.UBounds = [1;1];
    [x_opt_hat, fhat, COUNTEVAL, STOPFLAG, OUT, BESTEVER] = cmaes('IRF_matching_objective',x_start,H0,cmaesOptions,IRF_empirical,IRF_weighting);
    x_opt_hat=BESTEVER.x;
@#endif

%get IRFs at the optimum and plot them
[fval, IRF_model]=IRF_matching_objective(x_opt_hat,IRF_empirical,IRF_weighting);

figure
subplot(1,4,1)
plot(1:options_.irf,IRF_empirical(:,1),1:options_.irf,IRF_model(:,1),1:options_.irf,IRF_quantiles(1,:,1),'r--',1:options_.irf,IRF_quantiles(1,:,2),'r--','LineWidth',2);
%shade(1:options_.irf,IRF_quantiles(1,:,1),1:options_.irf,IRF_quantiles(1,:,2),'FillType',[1 2;2 1]);
title('Public spending')
subplot(1,4,2)
plot(1:options_.irf,IRF_empirical(:,2),1:options_.irf,IRF_model(:,2),1:options_.irf,IRF_quantiles(2,:,1),'r--',1:options_.irf,IRF_quantiles(2,:,2),'r--','LineWidth',2);
title('Output')
legend('Empirical','Model')
subplot(1,4,3)
plot(1:options_.irf,IRF_empirical(:,3),1:options_.irf,IRF_model(:,3),1:options_.irf,IRF_quantiles(3,:,1),'r--',1:options_.irf,IRF_quantiles(3,:,2),'r--','LineWidth',2);
title('Investment')
legend('Empirical','Model')
subplot(1,4,4)
plot(1:options_.irf,IRF_empirical(:,4),1:options_.irf,IRF_model(:,4),1:options_.irf,IRF_quantiles(4,:,1),'r--',1:options_.irf,IRF_quantiles(4,:,2),'r--','LineWidth',2);
title('Consumption')
legend('Empirical','Model')
